/* nylon - (socks{4,5}) proxy
 * http://mesh.eecs.umich.edu/projects/nylon
 * marius aamodt eriksen (marius@umich.edu)
 * utility functions */

#include "nylon.h"

#if __STDC__
#include <stdarg.h>
#else
#include <varargs.h>
#endif /* __STDC__ */

#define IFMAX IFNAMSIZ
#define DEFAULT_ALLOC 8

extern int verbose;

void
#if __STDC__
printv(int level, const char *fmt, ...)
#else
printv(level, fmt, va_alis)
int level;
char *fmt;
va_dcl
#endif /* __STDC__ */
{
	va_list ap;
#if __STDC__  
	va_start(ap, fmt);
#else
	va_start(ap);
#endif /* __STDC__ */	
	if (fmt && (level <= verbose)) {
		fprintf(stderr, "nylon: ");
		vfprintf(stderr, fmt, ap);
	}
	va_end(ap);
}

struct in_addr
get_ip_from_if(char *inf)
{
	struct ifreq ifr;
	struct sockaddr_in *sin;
	int bogusfd;
	char ifip[14];
	
	strlcpy(ifr.ifr_name, inf, IFMAX);
	bogusfd = socket(AF_INET, SOCK_STREAM, 0);
	ioctl(bogusfd, SIOCGIFADDR, &ifr);
	sin = (struct sockaddr_in *)&ifr.ifr_addr;
	/* display... -- XXX move this elsewhere */
	if (verbose) {
		inet_ntop(AF_INET, &sin->sin_addr, ifip, sizeof(ifip));
		printv(1, "binding to address %s\n", ifip);
	}
	return sin->sin_addr;
}

/* XXX strsep (doh!) */
char**
expanda(const char *in)
{
	char **ret = NULL, *a;
	char tmp;
	unsigned i, j, k, ac;

	if (in == NULL) return (char**)NULL;

	ac = i = k = 0;
	a = (char*)malloc(strlen(in) + 1);
	strlcpy(a, in, strlen(in) + 1);

	while (i < strlen(a)) {
		if ((k % DEFAULT_ALLOC) == 0) 
			ret = (char**)realloc(ret, sizeof(char*) * DEFAULT_ALLOC * (++ac));
		for (; a[i] && isspace(a[i]); ++i);
		if (i >= strlen(a)) goto fin;
		for (j = i; a[j] && !isspace(a[j]); ++j);
		tmp = a[j];
		a[j] = '\0';
		ret[k] = (char*)malloc(strlen(a + i) + 1);
		strlcpy(ret[k], a + i, strlen(a + i) + 1);
		a[j] = tmp;
		i = j;
		++k;
	}
	/* XXX */
 fin:
	if ((k % DEFAULT_ALLOC) == 0) 
		ret = (char**)realloc(ret, sizeof(char*)*DEFAULT_ALLOC * ac + 1);
	ret[k] = NULL;
	return ret;
}

void
make_ipchain(struct allow_head *ah, char **ip)
{
	struct ip_chain *node;
	unsigned i;

	if (ip == NULL)
		return;

	while (*ip) {
		node = (struct ip_chain*)malloc(sizeof(struct ip_chain));
		for (i=0; (*ip)[i] && ((*ip)[i] != '/'); i++);
		node->mask = 0xFFFFFFFF;
		if (i != strlen(*ip)) 
			node->mask <<= 32-atoi(*ip+i+1);
		(*ip)[i] = '\0';
		node->ip = inet_addr(*ip);
		TAILQ_INSERT_TAIL(ah, node, next);
		++ip; 
	}
	
	if (verbose) {
		printv(3, "ip chain:\n");
		TAILQ_FOREACH(node, ah, next) {
			struct in_addr in;
			in.s_addr = node->ip;
			printv(3, "%08lx (%s) / %08lx\n", node->ip, inet_ntoa(in), node->mask);
		}
	}
}

int
accept_ip(struct conf *c, struct sockaddr_in *in)
{
	int a = 0;
 	struct ip_chain *node;
	struct allow_head *ip_list[] = {(struct allow_head*)&c->ip_deny,
									&c->ip_allow,
									NULL};
	struct allow_head **il = ip_list;

	do {
		TAILQ_FOREACH(node, *il, next) {
			in_addr_t addr_in;
			in_addr_t addr_node;
			struct in_addr node_in, mask_in;

			node_in.s_addr = node->ip;
			mask_in.s_addr = htonl(node->mask);

			addr_in = in->sin_addr.s_addr & htonl(node->mask);
			addr_node = node->ip & htonl(node-> mask);
			
			printv(3, "in addr: %08lx : %s\n", in->sin_addr.s_addr,
				   inet_ntoa((struct in_addr)in->sin_addr));
			printv(3, "   addr: %08lx : %s\n", node->ip,
				   inet_ntoa(node_in));
			printv(3, "   mask: %08lx : %s\n", node->mask,
				   inet_ntoa(mask_in));			

			if (addr_in == addr_node) {
				if (!a)
					return 0;
				else
					return 1;
			}
		}
		a = 1;
	} while (*(++il));

	if (TAILQ_EMPTY(&c->ip_allow))
		return 1;

	return 0;
}

void
print_dump (u_char *buf, int len)
{
	int i, j, goback;

	printf("nylon: ");

	for (i = 0; i < len; ++i) {
		printf("%02x ", buf[i]);
		if ((goback = i % 16) == 15 || i == len - 1) {
			for (j = 15 - goback; j >= 0; --j) printf("   ");
			for (j = i - goback; j <= i; ++j)
				if (buf[j] > 31 && buf[j] < 127)
					printf("%c", buf[j]);
				else
					printf(".");
			if (i != len - 1)
				printf("\nnylon: ");
		} 
	}
	printf("\n");
}


/* copyright (c) 2001  
 * marius aamodt eriksen
 * all rights reserved
 * 
 * permission is granted to use, copy, create derivative works and
 * redistribute this software and such derivative works for any purpose,
 * so long as the name of the author is not used in any advertising or 
 * publicity pertaining to the use or distribution of this software 
 * without specific, written prior authorization.  if the above copyright 
 * notice or any other identification of the author is included in any 
 * copy of any portion of this software, then the disclaimer below must 
 * also be included.
 * 
 * this software is provided as is, without representation from the
 * the author as to its fitness for any purpose, and withoutwarranty by 
 * the author of any kind, either express or implied, including without 
 * limitation the implied warranties of merchantability and fitness for 
 * a particular purpose.  the author shall not be liable for any damages,
 * including special, indirect, incidental, or consequential damages,
 * with respect to any claim arising out or in connection with the use
 * of the software, even if it has been or is hereafter advised of the
 * possibility of such damages. */
